@@ -4,7 +4,7 @@ RDF::Trine::Store::DBI - Persistent RDF storage based on DBI
 
 =head1 VERSION
 
-This document describes RDF::Trine::Store::DBI version 1.011
+This document describes RDF::Trine::Store::DBI version 1.014
 
 =head1 SYNOPSIS
 
@@ -65,7 +65,7 @@ use RDF::Trine::Store::DBI::Pg;
 
 our $VERSION;
 BEGIN {
-	$VERSION	= "1.011";
+	$VERSION	= "1.014";
 	my $class	= __PACKAGE__;
 	$RDF::Trine::Store::STORE_CLASSES{ $class }	= $VERSION;
 }
@@ -214,7 +214,7 @@ sub nuke {
 	my $self	= shift;
 	my $dbh		= $self->dbh;
 	my $name	= $self->model_name;
-	my $id		= _mysql_hash( $name );
+	my $id		= $self->_mysql_hash( $name );
 	my $l		= Log::Log4perl->get_logger("rdf.trine.store.dbi");
 	
 	$dbh->do( "DROP TABLE Statements${id};" ) || do { $l->trace( $dbh->errstr ); return };
@@ -1200,6 +1200,9 @@ using the same algorithm that Redland's mysql storage backend uses.
 
 sub _mysql_hash;
 sub _mysql_hash_pp {
+	if (ref($_[0])) {
+		my $self = shift;
+	}
 	my $data	= encode('utf8', shift);
 	my @data	= unpack('C*', md5( $data ));
 	my $sum		= Math::BigInt->new('0');
@@ -1261,7 +1264,7 @@ sub _mysql_node_hash {
 		return;
 	}
 	my $hash;
-	$hash	= _mysql_hash( $data );
+	$hash	= $self->_mysql_hash( $data );
 	return $hash;
 }
 
@@ -1274,7 +1277,7 @@ Returns the name of the Statements table.
 sub statements_table {
 	my $self	= shift;
 	my $model	= $self->model_name;
-	my $id		= _mysql_hash( $model );
+	my $id		= $self->_mysql_hash( $model );
 	my $prefix	= $self->{statements_table_prefix};
 	return join('', $prefix, $id);
 }
@@ -1326,7 +1329,7 @@ sub make_private_predicate_view {
 	my $oldtable	= $self->statements_table;
 	my $oldpre		= $self->statements_prefix;
 	my $model		= $self->model_name;
-	my $id			= _mysql_hash( $model );
+	my $id			= $self->_mysql_hash( $model );
 	
 	my $stable		= join('', $prefix, $oldpre, $id);
 	my $predlist	= join(', ', map { $self->_mysql_node_hash( $_ ) } (@preds));
@@ -1358,7 +1361,7 @@ sub _debug {
 	my $self	= shift;
 	my $dbh		= $self->{dbh};
 	my $name	= $self->model_name;
-	my $id		= _mysql_hash( $name );
+	my $id		= $self->_mysql_hash( $name );
 	my $table	= 'Statements' . $id;
 	my $sth		= $dbh->prepare( "SELECT * FROM $table" );
 	$sth->execute;
@@ -1379,7 +1382,7 @@ sub init {
 	my $self	= shift;
 	my $dbh		= $self->dbh;
 	my $name	= $self->model_name;
-	my $id		= _mysql_hash( $name );
+	my $id		= $self->_mysql_hash( $name );
 	my $l		= Log::Log4perl->get_logger("rdf.trine.store.dbi");
 	
 	unless ($self->_table_exists("Literals")) {
@@ -1444,7 +1447,7 @@ sub _cleanup {
 	my $self	= shift;
 	if (my $dbh = $self->dbh) {
 		my $name	= $self->{model_name};
-		my $id		= _mysql_hash( $name );
+		my $id		= $self->_mysql_hash( $name );
 		if ($self->{ remove_store }) {
 			$dbh->do( "DROP TABLE `Statements${id}`;" );
 			$dbh->do( "DELETE FROM Models WHERE Name = ?", undef, $name );
